local L = LibStub("AceLocale-3.0"):GetLocale("ExtMailUI", true);
local MAILBOX_OPENING = false;
local SUPPRESS_SEARCH_ONCHANGE_UPDATE = false;

EXTMAILUI_DATA = {};

-- ==================================================
--  Main OnLoad handler
-- ==================================================
function ExtMailUI_OnLoad(self)
    self.ITEMS_PER_ROW = 13;
    self.TOTAL_ITEMS_VISIBLE = 50;
    self.Hooks = {};
    self.HalfMode = false;
    self.Version = GetAddOnMetadata("ExtMailUI", "Version");

    self:RegisterEvent("ADDON_LOADED");
    MailFrame:HookScript("OnShow", ExtMailUI_MailFrame_OnShow);
    InboxFrame:HookScript("OnShow", ExtMailUI_InboxFrame_OnShow);
    SendMailFrame:HookScript("OnShow", ExtMailUI_SendMailFrame_OnShow);
    
    self.Hooks["InboxFrame_Update"] = InboxFrame_Update;
    InboxFrame_Update = ExtMailUI_Hook_InboxFrame_Update;
    self.Hooks["SendMailFrame_Update"] = SendMailFrame_Update;
    SendMailFrame_Update = ExtMailUI_Hook_SendMailFrame_Update;
    self.Hooks["InboxFrameItem_OnEnter"] = InboxFrameItem_OnEnter;
    InboxFrameItem_OnEnter = ExtMailUI_Hook_InboxFrameItem_OnEnter;
    
    --SLASH_EXTMAILUI1 = "/emui";
    --SlashCmdList["EXTMAILUI"] = ExtMailUI_CommandHandler;
    
    MailFrameExtMailUITab:SetText(L["ADDON_TITLE"]);
    PanelTemplates_DeselectTab(MailFrameExtMailUITab);
end

-- ==================================================
--  Event handler
-- ==================================================
function ExtMailUI_OnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addon = ...;
        if (addon == "ExtMailUI") then
            ExtMailUI_Setup();
        elseif (addon == "Postal") then
            ExtMailUI_Postal_OnLoad();
        end
    end
end

-- ==================================================
--  Post-load setup: Load configuration settings,
--  perform MailFrame modifications, etc.
-- ==================================================
function ExtMailUI_Setup()
    -- initialize config if not already defined
    if (not EXTMAILUI_DATA['config']) then
        EXTMAILUI_DATA['config'] = {
            enableAutoLoot = false,
            halfWidthMode = false,
            loadMessage = false,
        };
    end

    ExtMailUI_RebuildInbox();
    ExtMailUI_SetHalfMode(EXTMAILUI_DATA['config'].halfWidthMode);
    
    if (EXTMAILUI_DATA['config'].loadMessage) then
        ExtMailUI_Message(string.format(L["LOADED_MESSAGE"], ExtendedMailUI.Version));
    end
end

-- ==================================================
--  Chat frame message output
-- ==================================================
function ExtMailUI_Message(text)
    DEFAULT_CHAT_FRAME:AddMessage("|cffc0b0a0<Extended Mail UI>|r " .. text);
end

-- ==================================================
--  Rebuild the inbox frame, and increase the height
--  of the send mail frame to match
-- ==================================================
function ExtMailUI_RebuildInbox()
    -- resize the main parent frame
    MailFrame:SetWidth(1050);   -- +662
    MailFrame:SetHeight(500);   -- +76
    
    -- resize the inbox frame, change the background texture and reposition the Open All button
    InboxFrame:SetWidth(1096);
    InboxFrame:SetHeight(588);
    InboxNextPageButton:Hide();
    InboxPrevPageButton:Hide();
    InboxFrameBg:SetWidth(1033);
    InboxFrameBg:SetHeight(431);
    InboxFrameBg:SetTexture("Interface/AddOns/ExtMailUI/textures/background");
    InboxFrameBg:SetTexCoord(0, 1, 0, 1);
    InboxFrameBg:SetVertexColor(0.85, 0.8, 0.75);
    InboxTitleText:ClearAllPoints();
    InboxTitleText:SetPoint("TOP", InboxFrame, "TOP", 0, -5);
    OpenAllMail:ClearAllPoints();
    OpenAllMail:SetPoint("TOPRIGHT", InboxFrame, "TOPRIGHT", -55, -29);

    -- move the previous and next buttons for half-width mode
    InboxPrevPageButton:ClearAllPoints();
    InboxPrevPageButton:SetPoint("CENTER", InboxFrame, "BOTTOMLEFT", 300, 114);
    InboxNextPageButton:ClearAllPoints();
    InboxNextPageButton:SetPoint("CENTER", InboxFrame, "BOTTOMRIGHT", -75, 114);
    
    -- increase the height of the send mail frame to match the new height of the inbox frame
    SendMailFrame:SetHeight(588);
    SendMailHorizontalBarLeft:ClearAllPoints();
    SendMailHorizontalBarLeft:SetPoint("BOTTOMLEFT", SendMailFrame, "BOTTOMLEFT", 2, 158);
    SendMailTitleText:ClearAllPoints();
    SendMailTitleText:SetPoint("TOP", SendMailFrame, "TOP", 0, -5);
    
    -- create text to display how many more messages are not displayed when the inbox is full
    local moreInboxItems = InboxFrame:CreateFontString("InboxFrameMoreMessages", "ARTWORK", "GameFontHighlight");
    moreInboxItems:SetWidth(180);
    moreInboxItems:SetHeight(60);
    moreInboxItems:SetJustifyH("CENTER");
    moreInboxItems:SetJustifyV("MIDDLE");
    moreInboxItems:SetPoint("CENTER", InboxFrame, "BOTTOMRIGHT", -180, 135);
    moreInboxItems:SetText("");
    
    -- add search box
    local editbox = CreateFrame("EditBox", "InboxFrameSearchBox", InboxFrame, "SearchBoxTemplate");
    editbox:SetWidth(200);
    editbox:SetHeight(24);
    editbox:SetPoint("TOPLEFT", InboxFrame, "TOPLEFT", 70, -29);
    editbox:SetAutoFocus(false);
    editbox:HookScript("OnTextChanged", ExtMailUI_Search_OnTextChanged);
    editbox:SetMaxLetters(30);
    
    ExtMailUI_AlterMailItemSlots();
    
    INBOXITEMS_TO_DISPLAY = ExtendedMailUI.TOTAL_ITEMS_VISIBLE;
end

-- ==================================================
--  Create additional mail slot elements and modify
--  the existing ones
-- ==================================================
function ExtMailUI_AlterMailItemSlots()
    local i;
    local leftMargin = 0;
    -- add a margin for the checkboxes created by Postal's Select module
    if (ExtMailUI_Postal_IsEnabled("Select")) then
        leftMargin = 20;
    end
    for i = 1, 50 do
        local mailItem = _G["MailItem" .. i];
        if (i <= ExtendedMailUI.TOTAL_ITEMS_VISIBLE) then
            if (not mailItem) then
                mailItem = CreateFrame("Frame", "MailItem" .. i, InboxFrame, "MailItemTemplate");
            end
            mailItem:Show();
            if (i > 1) then
                -- anchor the top slot of each column to the top slot of the previous column
                if ((i % ExtendedMailUI.ITEMS_PER_ROW) == 1) then
                    mailItem:SetPoint("TOPLEFT", "MailItem" .. (i - ExtendedMailUI.ITEMS_PER_ROW), "TOPRIGHT", 20 + leftMargin, 0);
                -- anchor others to the slot above them
                else
                    mailItem:SetPoint("TOPLEFT", "MailItem" .. (i - 1), "BOTTOMLEFT", 0, 0);
                end
            else
                -- re-anchor the first mail slot
                mailItem:SetPoint("TOPLEFT", InboxFrame, "TOPLEFT", 13 + leftMargin, -70);
            end
            ExtMailUI_AlterMailItemSlot("MailItem" .. i);
        else
            if (mailItem) then
                mailItem:Hide();
            end
        end
    end
end

-- ==================================================
--  Individual mail slot modifications
-- ==================================================
function ExtMailUI_AlterMailItemSlot(frameName)
    local frame = _G[frameName];
    local buttonSlot = _G[frameName .. "ButtonSlot"];
    local sender = _G[frameName .. "Sender"];
    local subject = _G[frameName .. "Subject"];
    local expires = _G[frameName .. "ExpireTime"];
    
    local widthMod = (ExtMailUI_Postal_IsEnabled("Select") and -20 or 0);
    
    frame:SetHeight(32);
    frame:SetWidth(240 + widthMod);
    frame.Button:SetWidth(23);
    frame.Button:SetHeight(23);
    buttonSlot:SetWidth(42);
    buttonSlot:SetHeight(42);
    
    -- because the textures for each slot are unnamed, we have to search for them by their width/height
    local i, r;
    for i, r in pairs({ frame:GetRegions() }) do
        if (r:IsObjectType("Texture")) then
            -- main slot textures
            if (r:GetHeight() > 30) then
                r:SetHeight(33);
                -- empty icon slot
                if (r:GetWidth() < 100) then
                    r:SetWidth(27);
                -- sender/subject area
                else
                    r:SetWidth(213 + widthMod);
                end
            -- lower border line
            else
                r:SetWidth(257 + widthMod);
            end
        end
    end
    
    -- shift the sender/subject up just a bit to better fit the smaller height
    sender:ClearAllPoints();
    sender:SetPoint("TOPLEFT", frame, "TOPLEFT", 34, 0);
    
    subject:SetWidth(188 + widthMod);

    -- and the expiration text
    expires:ClearAllPoints();
    expires:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -4, 0);
end

-- ==================================================
--  Override for opening the mailbox
-- ==================================================
function ExtMailUI_MailFrame_OnShow(self)
    if (IsAddOnLoaded("Postal")) then
        ExtMailUI_FixPostalButtons();
        ExtMailUI_PostalSelect_Incompatible();
        ExtMailUI_Postal_PostHooks();
        ExtMailUI_PostalSelect_UpdateCheckboxes();
        ExtMailUI_PostalDNW_UpdateReturnIcons();
    end
    MAILBOX_OPENING = true;
    SUPPRESS_SEARCH_ONCHANGE_UPDATE = true;
end

-- ==================================================
--  Override for opening/switching to the Inbox tab
-- ==================================================
function ExtMailUI_InboxFrame_OnShow(self)
    -- we have to adjust the width of the frames every time because the send mail frame is the original width
    MailFrame:SetWidth(ExtendedMailUI.HalfMode and 540 or 1050);
    InboxFrame:SetWidth(ExtendedMailUI.HalfMode and 586 or 1096);
    InboxFrameBg:SetWidth(ExtendedMailUI.HalfMode and 523 or 1033);
    -- reposition other frames that may be open when switching mailbox tabs
    UpdateUIPanelPositions(MailFrame);
    ExtMailUI_PostalSelect_Incompatible();
    ExtMailUI_AlterMailItemSlots();
end

-- ==================================================
--  Override for opening/switching to the
--  Send Mail tab
-- ==================================================
function ExtMailUI_SendMailFrame_OnShow(self)
    -- set parent frame back to original width
    MailFrame:SetWidth(338);
    -- reposition other frames that may be open when switching mailbox tabs
    UpdateUIPanelPositions(MailFrame);
end

-- ==================================================
--  Override for updating the inbox contents
-- ==================================================
function ExtMailUI_Hook_InboxFrame_Update()
    --HOOKS["InboxFrame_Update"]();
    -- this is all mostly copy-pasted from the original InboxFrame_Update() function, but with a couple minor additions/changes to the item iteration,
    -- hence why the original hooked function isn't being called
    
    local numItems, totalItems = GetInboxNumItems();
    local index = ((InboxFrame.pageNum - 1) * INBOXITEMS_TO_DISPLAY) + 1;
    local packageIcon, stationeryIcon, sender, subject, money, CODAmount, daysLeft, itemCount, wasRead, x, y, canReply, isGM, firstItemQuantity, firstItemLink;
    local slotItem, icon, button, expireTime, senderText, subjectText, buttonIcon;
    
    for i = 1, INBOXITEMS_TO_DISPLAY do
        if (index <= numItems) then
            -- Setup mail item
            packageIcon, stationeryIcon, sender, subject, money, CODAmount, daysLeft, itemCount, wasRead, x, y, canReply, isGM, firstItemQuantity, firstItemLink = GetInboxHeaderInfo(index);

            local searchIsMatch = ExtMailUI_IsSearchMatch(InboxFrameSearchBox:GetText(), sender, subject);
            
            -- Set icon
            if ( packageIcon ) and ( not isGM ) then
                icon = packageIcon;
            else
                icon = stationeryIcon;
            end
            
            -- If no sender set it to "Unknown"
            if ( not sender ) then
                sender = UNKNOWN;
            end
            
            slotItem = _G["MailItem" .. i];
            button = _G["MailItem" .. i .. "Button"];
            button:Show();
            button.index = index;
            button.hasItem = itemCount;
            button.itemCount = itemCount;
            SetItemButtonCount(button, firstItemQuantity);
            if (firstItemQuantity) then
                SetItemButtonQuality(button, select(3, GetItemInfo(firstItemLink)), firstItemLink);
            else
                button.IconBorder:Hide();
                button.IconOverlay:Hide();
            end
            
            buttonIcon = _G["MailItem"..i.."ButtonIcon"];
            buttonIcon:SetTexture(icon);
            subjectText = _G["MailItem"..i.."Subject"];
            subjectText:SetText(((CODAmount > 0) and "|cffff8000[C.O.D]|r " or "") .. subject);     -- COD indication shows in here now
            senderText = _G["MailItem"..i.."Sender"];
            senderText:SetText(sender);
            
            -- If hasn't been read color the button yellow
            if (searchIsMatch) then
                if (wasRead) then
                    senderText:SetTextColor(0.75, 0.75, 0.75);
                    subjectText:SetTextColor(0.75, 0.75, 0.75);
                    _G["MailItem" .. i .. "ButtonSlot"]:SetVertexColor(0.5, 0.5, 0.5);
                    SetDesaturation(buttonIcon, true);
                    button.IconBorder:SetVertexColor(0.5, 0.5, 0.5);
                else
                    senderText:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
                    subjectText:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
                    _G["MailItem" .. i .. "ButtonSlot"]:SetVertexColor(1.0, 0.82, 0);
                    SetDesaturation(buttonIcon, false);
                end
                slotItem:SetAlpha(1);
            else
                senderText:SetTextColor(0.5, 0.5, 0.5);
                subjectText:SetTextColor(0.5, 0.5, 0.5);
                slotItem:SetAlpha(0.5);
                SetDesaturation(buttonIcon, true);
            end
            -- Format expiration time
            if ( daysLeft >= 1 ) then
                daysLeft = GREEN_FONT_COLOR_CODE..format(DAYS_ABBR, floor(daysLeft)).." "..FONT_COLOR_CODE_CLOSE;
            else
                daysLeft = RED_FONT_COLOR_CODE..SecondsToTime(floor(daysLeft * 24 * 60 * 60))..FONT_COLOR_CODE_CLOSE;
            end
            expireTime = _G["MailItem"..i.."ExpireTime"];
            expireTime:SetText(daysLeft);
            -- Set expiration time tooltip
            if ( InboxItemCanDelete(index) ) then
                expireTime.tooltip = TIME_UNTIL_DELETED;
            else
                expireTime.tooltip = TIME_UNTIL_RETURNED;
            end
            expireTime:Show();
            -- Is a C.O.D. package
            if ( CODAmount > 0 ) then
                --_G["MailItem"..i.."ButtonCOD"]:Show();
                --_G["MailItem"..i.."ButtonCODBackground"]:Show();
                _G["MailItem"..i.."ButtonCOD"]:Hide();              -- moved COD indication to the subject line, so we don't need to show this anymore
                _G["MailItem"..i.."ButtonCODBackground"]:Hide();
                button.cod = CODAmount;
            else
                _G["MailItem"..i.."ButtonCOD"]:Hide();
                _G["MailItem"..i.."ButtonCODBackground"]:Hide();
                button.cod = nil;
            end
            -- Contains money
            if ( money > 0 ) then
                button.money = money;
            else
                button.money = nil;
            end
            -- Set highlight
            if ( InboxFrame.openMailID == index ) then
                button:SetChecked(true);
                SetPortraitToTexture("OpenMailFrameIcon", stationeryIcon);
            else
                button:SetChecked(false);
            end
            
            -- tag the button object with some of the information pulled from the GetInboxHeaderInfo return
            button.sender = sender;
            button.subject = subject;
            button.isGM = isGM;
            button.wasRead = wasRead;
        else
            -- Clear everything
            _G["MailItem"..i.."Button"]:Hide();
            _G["MailItem"..i.."Sender"]:SetText("");
            _G["MailItem"..i.."Subject"]:SetText("");
            _G["MailItem"..i.."ExpireTime"]:Hide();
        end
        index = index + 1;
    end
    
    if (ExtendedMailUI.HalfMode) then
        InboxPrevPageButton:Show();
        InboxNextPageButton:Show();
        if ( InboxFrame.pageNum == 1 ) then
            InboxPrevPageButton:Disable();
        else
            InboxPrevPageButton:Enable();
        end
        if ( (InboxFrame.pageNum * INBOXITEMS_TO_DISPLAY) < numItems ) then
            InboxNextPageButton:Enable();
        else
            InboxNextPageButton:Disable();
        end
    else
        InboxPrevPageButton:Hide();
        InboxPrevPageButton:Disable();
        InboxNextPageButton:Hide();
        InboxNextPageButton:Disable();
    end
    if (ExtendedMailUI.HalfMode) then
        -- having the search box means there's no room for this in half-width mode now
        InboxTooMuchMail:Hide();
    else
        if (not ExtMailUI_Postal_IsEnabled("Select")) then
            if ( totalItems > numItems) then
                InboxTooMuchMail:Show();
            else
                InboxTooMuchMail:Hide();
            end
        end
    end
    
    ExtMailUI_PostalSelect_UpdateCheckboxes();
    ExtMailUI_PostalDNW_UpdateReturnIcons();
    
    if ((totalItems > numItems) and (not ExtendedMailUI.HalfMode)) then
        InboxFrameMoreMessages:Show();
        InboxFrameMoreMessages:SetText((totalItems - numItems) .. " more messages not shown");
    else
        InboxFrameMoreMessages:Hide();
    end
    
    -- if 'auto loot' is enabled, the auto loot key is being held down, and the mailbox is
    -- just being opened, immediately simulate clicking the Open All button
    if (MAILBOX_OPENING) then
        if (EXTMAILUI_DATA['config'].enableAutoLoot) then
            if (IsModifiedClick("AUTOLOOTTOGGLE")) then
                if (PostalOpenAllButton) then
                    PostalOpenAllButton:Click();
                else
                    OpenAllMail:Click();
                end
            end
        end
        MAILBOX_OPENING = false;
    end
    if (SUPPRESS_SEARCH_ONCHANGE_UPDATE) then
        SUPPRESS_SEARCH_ONCHANGE_UPDATE = false;
    end
end

-- ==================================================
--  Override for showing inbox item tooltips
-- ==================================================
function ExtMailUI_Hook_InboxFrameItem_OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    
    -- add sender and subject to the tooltip now
    GameTooltip:AddLine("|cffffffffSent by " .. self.sender .. "|r");
    if (self.subject) then
        GameTooltip:AddLine("|cff0080ffSubject: " .. self.subject .. "|r");
    end
    
    if ( self.hasItem ) then
        if ( self.itemCount == 1) then
            local hasCooldown, speciesID, level, breedQuality, maxHealth, power, speed, name = GameTooltip:SetInboxItem(self.index);
            if(speciesID and speciesID > 0) then
                BattlePetToolTip_Show(speciesID, level, breedQuality, maxHealth, power, speed, name);
            end
        else
            GameTooltip:AddLine(" ");
            GameTooltip:AddLine(MAIL_MULTIPLE_ITEMS.." ("..self.itemCount..")");
        end
    end
    if (self.money) then
        if ( self.hasItem ) then
            GameTooltip:AddLine(" ");
        end
        GameTooltip:AddLine(ENCLOSED_MONEY, nil, nil, nil, true);
        SetTooltipMoney(GameTooltip, self.money);
        SetMoneyFrameColor("GameTooltipMoneyFrame1", "white");
    elseif (self.cod) then
        if ( self.hasItem ) then
            GameTooltip:AddLine(" ");
        end
        GameTooltip:AddLine(COD_AMOUNT, nil, nil, nil, true);
        SetTooltipMoney(GameTooltip, self.cod);
        if (self.cod > GetMoney()) then
            SetMoneyFrameColor("GameTooltipMoneyFrame1", "red");
        else
            SetMoneyFrameColor("GameTooltipMoneyFrame1", "white");
        end
    end
    GameTooltip:Show();
end

-- ==================================================
--  Hook for updating the Send Mail tab
-- ==================================================
function ExtMailUI_Hook_SendMailFrame_Update()
    ExtendedMailUI.Hooks["SendMailFrame_Update"]();
    
    -- Update the item(s) being sent
    local itemCount = 0;
    local itemTitle;
    local gap = false;
    local last = 0;
    for i=1, ATTACHMENTS_MAX_SEND do
        --local sendMailAttachmentButton = SendMailFrame.SendMailAttachments[i];

        if HasSendMailItem(i) then
            itemCount = itemCount + 1;

            --local itemName, itemID, itemTexture, stackCount, quality = GetSendMailItem(i);
            --sendMailAttachmentButton:SetNormalTexture(itemTexture or "Interface\\Icons\\INV_Misc_QuestionMark");
            --SetItemButtonCount(sendMailAttachmentButton, stackCount or 0);
            --SetItemButtonQuality(sendMailAttachmentButton, quality, itemID);
        
            -- determine what a name for the message in case it doesn't already have one
            --if not itemTitle and itemName then
            --  if stackCount <= 1 then
            --      itemTitle = itemName;
            --  else
            --      itemTitle = itemName.." ("..stackCount..")";
            --  end
            --end

            if last + 1 ~= i then
                gap = true;
            end
            last = i;
        --else
            --sendMailAttachmentButton:SetNormalTexture(nil);
            --SetItemButtonCount(sendMailAttachmentButton, 0);
            --SetItemButtonQuality(sendMailAttachmentButton, nil);
        end
    end

    -- Determine how many rows of attachments to show
    local itemRowCount = 1;
    local temp = last;
    while ((temp > ATTACHMENTS_PER_ROW_SEND) and (itemRowCount < ATTACHMENTS_MAX_ROWS_SEND)) do
        itemRowCount = itemRowCount + 1;
        temp = temp - ATTACHMENTS_PER_ROW_SEND;
    end
    if (not gap and (temp == ATTACHMENTS_PER_ROW_SEND) and (itemRowCount < ATTACHMENTS_MAX_ROWS_SEND)) then
        itemRowCount = itemRowCount + 1;
    end
    if (SendMailFrame.maxRowsShown and (last > 0) and (itemRowCount < SendMailFrame.maxRowsShown)) then
        itemRowCount = SendMailFrame.maxRowsShown;
    else
        SendMailFrame.maxRowsShown = itemRowCount;
    end

    -- Compute sizes
    local cursorx = 0;
    local cursory = itemRowCount - 1;
    local marginxl = 8 + 6;
    local marginxr = 40 + 6;
    local areax = SendMailFrame:GetWidth() - marginxl - marginxr;
    local iconx = SendMailAttachment1:GetWidth() + 2;
    local icony = SendMailAttachment1:GetHeight() + 2;
    local gapx1 = floor((areax - (iconx * ATTACHMENTS_PER_ROW_SEND)) / (ATTACHMENTS_PER_ROW_SEND - 1));
    local gapx2 = floor((areax - (iconx * ATTACHMENTS_PER_ROW_SEND) - (gapx1 * (ATTACHMENTS_PER_ROW_SEND - 1))) / 2);
    local gapy1 = 5;
    local gapy2 = 6;
    local areay = (gapy2 * 2) + (gapy1 * (itemRowCount - 1)) + (icony * itemRowCount);
    local indentx = marginxl + gapx2;
    local indenty = 170 + gapy2 + icony;
    local tabx = (iconx + gapx1) - 2; --this magic number changes the attachment spacing
    local taby = (icony + gapy1);
    local scrollHeight = 325 - areay;

    -- Resize the scroll frame
    SendMailScrollFrame:SetHeight(scrollHeight);
    SendMailScrollChildFrame:SetHeight(scrollHeight);
    SendMailHorizontalBarLeft2:SetPoint("TOPLEFT", "SendMailFrame", "BOTTOMLEFT", 2, 184 + areay);
    SendScrollBarBackgroundTop:SetHeight(min(scrollHeight, 256));
    SendScrollBarBackgroundTop:SetTexCoord(0, 0.484375, 0, min(scrollHeight, 256) / 256);
    SendStationeryBackgroundLeft:SetHeight(min(scrollHeight, 274));
    SendStationeryBackgroundLeft:SetTexCoord(0, 1.0, 0, min(scrollHeight, 274) / 274);
    SendStationeryBackgroundRight:SetHeight(min(scrollHeight, 274));
    SendStationeryBackgroundRight:SetTexCoord(0, 1.0, 0, min(scrollHeight, 274) / 274);
    SendStationeryBackgroundLeft:SetTexture("Interface/Stationery/stationerytest1");
    SendStationeryBackgroundRight:SetTexture("Interface/Stationery/stationerytest2");
end

-- ==================================================
--  Enables/disables half-width mode
-- ==================================================
function ExtMailUI_SetHalfMode(state)
    -- if turning on half-width mode, disable Postal's Select module and display an alert in the chat frame
    if (state) then
        if (ExtMailUI_Postal_IsEnabled("Select")) then
            ExtMailUI_Message(L["POSTAL_SELECT_DISABLED"]);
            Postal:GetModule("Select"):Disable();
        end
    end
    InboxFrame.pageNum = 1;
    ExtendedMailUI.HalfMode = state;
    ExtendedMailUI.TOTAL_ITEMS_VISIBLE = state and 25 or 50;
    INBOXITEMS_TO_DISPLAY = ExtendedMailUI.TOTAL_ITEMS_VISIBLE;
    if (InboxFrame:IsShown()) then
        ExtMailUI_InboxFrame_OnShow(InboxFrame);
    end
    ExtMailUI_Hook_InboxFrame_Update();
    ExtMailUI_FixPostalButtons();
    UpdateUIPanelPositions(MailFrame);
end

-- ==================================================
--  Slash command handler
-- ==================================================
function ExtMailUI_CommandHandler(cmd)
    ExtMailUI_ShowMainConfig();
end

-- ==================================================
--  Clicking the tab
-- ==================================================
function ExtMailUI_TabClick()
    ExtMailUI_ShowMainConfig();
end

-- ==================================================
--  Search text change handler
-- ==================================================
function ExtMailUI_Search_OnTextChanged()
    if (SUPPRESS_SEARCH_ONCHANGE_UPDATE) then return; end
    InboxFrame_Update();
end

-- ==================================================
--  Returns whether a mail shot should be shown or
--  hidden based on search text, sender and subject
-- ==================================================
function ExtMailUI_IsSearchMatch(searchString, sender, subject)
    local search = string.lower(string.trim(searchString));
    if (string.len(searchString) > 0) then
        if ((not string.find(string.lower(sender), search, 1, true)) and (not string.find(string.lower(subject), search, 1, true))) then
            return false;
        end
    end
    return true;
end
