local L = LibStub("AceLocale-3.0"):GetLocale("ExtMailUI", true);

local hookedSelect = false;

-- ==================================================
--  Handler for when Postal finishes loading
-- ==================================================
function ExtMailUI_Postal_OnLoad()
    ExtendedMailUI.Hooks["Postal.ToggleModule"] = Postal.ToggleModule;
    Postal.ToggleModule = ExtMailUI_Hook_Postal_ToggleModule;
end

-- ==================================================
--  Repositions Postal's Open, Return and Open All
--  buttons
-- ==================================================
function ExtMailUI_FixPostalButtons()
    if (not IsAddOnLoaded("Postal")) then return; end
    -- move the Open All button (and its menu) to the top right corner
    if (PostalOpenAllButton) then
        PostalOpenAllButton:ClearAllPoints();
        PostalOpenAllButton:SetPoint("TOPRIGHT", InboxFrame, "TOPRIGHT", -80, -29);
    end
    
    --if (ExtMailUI_Postal_IsEnabled("Select")) then
    --    -- if in half-width mode, move the Select module's Open and Return buttons to the top left corner
    --    if (ExtendedMailUI.HalfMode) then
    --        PostalSelectOpenButton:ClearAllPoints();
    --        PostalSelectOpenButton:SetPoint("LEFT", InboxFrame, "TOPLEFT", 100, -42);
    --        PostalSelectReturnButton:ClearAllPoints();
    --        PostalSelectReturnButton:SetPoint("LEFT", PostalSelectOpenButton, "RIGHT", 5, 0);
    --    -- otherwise keep them in the top center as they normally are
    --    else
    --        PostalSelectOpenButton:ClearAllPoints();
    --        PostalSelectOpenButton:SetPoint("RIGHT", InboxFrame, "TOP", 0, -42);
    --        PostalSelectReturnButton:ClearAllPoints();
    --        PostalSelectReturnButton:SetPoint("LEFT", InboxFrame, "TOP", 5, -42);
    --    end
    --end
end

function ExtMailUI_Postal_PostHooks()
    --if (ExtMailUI_Postal_IsEnabled("Select") and (not hookedSelect)) then
    --    ExtMailUI_Message("Hooking Postal.Select.HandleSelect");
    --    --local smod = Postal:GetModule("Select");
    --    --if (smod) then
    --    --    ExtendedMailUI.Hooks["PostalSelect.HandleSelect"] = smod.HandleSelect;
    --    --    smod.HandleSelect = function(mode)
    --    --        ExtendedMailUI.Hooks["PostalSelect.HandleSelect"](mode);
    --    --        ExtMailUI_PostalSelect_HandleSelect(mode);
    --    --    end
    --    --end
    --    PostalSelectOpenButton:SetScript("OnClick", function() ExtMailUI_PostalSelect_HandleSelect(1); end);
    --    PostalSelectReturnButton:SetScript("OnClick", function() ExtMailUI_PostalSelect_HandleSelect(2); end);
    --    hookedSelect = true;
    --end
end

-- ==================================================
--  Hook Postal's module toggling from the menu,
--  so we can update the inbox after toggling the
--  Select module
-- ==================================================
function ExtMailUI_Hook_Postal_ToggleModule(dropdownbutton, arg1, arg2, checked)
    ExtendedMailUI.Hooks["Postal.ToggleModule"](dropdownbutton, arg1, arg2, checked);
    if (arg1 == "Select") then
        -- prevent the Select module from being turned on when half-width mode is enabled
        if (ExtMailUI_Postal_IsEnabled("Select")) then
            ExtMailUI_Message(L["POSTAL_MODULE_INCOMPATIBLE"]);
            Postal:GetModule("Select"):Disable();
            InboxFrame_Update();
        end
        -- force a full update
        ExtMailUI_AlterMailItemSlots();
        ExtMailUI_PostalSelect_UpdateCheckboxes();
        ExtMailUI_PostalDNW_UpdateReturnIcons();
        ExtMailUI_Postal_PostHooks();
    end
end

-- ==================================================
--  An easy check to see if a specific Postal module
--  (or Postal itself) is enabled
-- ==================================================
function ExtMailUI_Postal_IsEnabled(moduleName)
    if (Postal) then
        local smod = Postal:GetModule(moduleName);
        if (smod) then
            return smod:IsEnabled();
        end
    end
    return false;
end

-- **************************************** MODULE HELPERS: SELECT ****************************************

-- ==================================================
--  Disables Postal's Select module and displays
--  an incompatibility warning
-- ==================================================
function ExtMailUI_PostalSelect_Incompatible()
    if (not ExtMailUI_Postal_IsEnabled("Select")) then return; end
    ExtMailUI_Message(L["POSTAL_SELECT_DISABLED"]);
    Postal:GetModule("Select"):Disable();
end

-- ==================================================
--  Main checkbox update branch
-- ==================================================
function ExtMailUI_PostalSelect_UpdateCheckboxes()
    if (ExtMailUI_Postal_IsEnabled("Select")) then
        ExtMailUI_PostalSelect_AddCheckboxes();
        ExtMailUI_PostalSelect_RefreshCheckboxes();
    else
        ExtMailUI_PostalSelect_RemoveCheckboxes();
    end
end

-- ==================================================
--  Create additional checkboxes for new mail slots
-- ==================================================
function ExtMailUI_PostalSelect_AddCheckboxes()
    local checkboxFunc = function(self) Postal:GetModule("Select"):ToggleMail(self) end
    local i;
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
		if (not _G["PostalInboxCB" .. i]) then
			local CB = CreateFrame("CheckButton", "PostalInboxCB" .. i, _G["MailItem" .. i], "OptionsCheckButtonTemplate");
			CB:SetID(i);
			CB:SetPoint("RIGHT", "MailItem" .. i, "LEFT", 1, -5);
			CB:SetWidth(24);
			CB:SetHeight(24);
			CB:SetHitRectInsets(0, 0, 0, 0);
			CB:SetScript("OnClick", checkboxFunc);
			local text = CB:CreateFontString("PostalInboxCB" .. i .. "Text", "BACKGROUND", "GameFontHighlightSmall");
			text:SetPoint("BOTTOM", CB, "TOP");
			text:SetText(i);
			CB.text = text;
		end
	end
end

-- ==================================================
--  Hide all checkboxes
-- ==================================================
function ExtMailUI_PostalSelect_RemoveCheckboxes()
    local i;
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
        local check = _G["PostalInboxCB" .. i];
		if (check) then
            check:Hide();
		end
	end
end

-- ==================================================
--  Refresh checkboxes
-- ==================================================
function ExtMailUI_PostalSelect_RefreshCheckboxes()
    local i;
    local numItems, totalItems = GetInboxNumItems();
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
		local index = i + ((InboxFrame.pageNum - 1) * ExtendedMailUI.TOTAL_ITEMS_VISIBLE);
        local check = _G["PostalInboxCB" .. i];
		if (check) then
            if (i <= numItems) then
                check:Show();
                check:SetID(index);
                check.text:SetText(index);
            else
                check:Hide();
            end
		end
	end
end

function ExtMailUI_PostalSelect_HandleSelect(mode)
    --ExtendedMailUI.Hooks["PostalSelect.HandleSelect"](mode);
    ExtMailUI_PostalSelect_RemoveCheckboxes();
    Postal:GetModule("Select"):HandleSelect(mode);
    ExtMailUI_Message("HandleSelect hook (" .. mode .. ")");
end

-- **************************************** MODULE HELPERS: DONOTWANT ****************************************

-- ==================================================
--  Main return/delete button update branch
-- ==================================================
function ExtMailUI_PostalDNW_UpdateReturnIcons()
    if (ExtMailUI_Postal_IsEnabled("DoNotWant")) then
        ExtMailUI_PostalDNW_AddReturnIcons();
        ExtMailUI_PostalDNW_RefreshReturnIcons();
    else
        ExtMailUI_PostalDNW_RemoveReturnIcons();
    end
end

-- ==================================================
--  Create new buttons for additional mail slots
-- ==================================================
function ExtMailUI_PostalDNW_AddReturnIcons()
    local i;
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
		local b = _G["MailItem" .. i .. "ExpireTime"];
		if not b.returnicon then
			b.returnicon = CreateFrame("BUTTON", nil, b)
			b.returnicon:SetPoint("TOPRIGHT", b, "BOTTOMRIGHT", -5, -1)
			b.returnicon:SetWidth(16)
			b.returnicon:SetHeight(16)
			b.returnicon.texture = b.returnicon:CreateTexture(nil, "BACKGROUND")
			b.returnicon.texture:SetAllPoints()
			b.returnicon.texture:SetTexCoord(1, 0, 0, 1) -- flips image left/right
			b.returnicon.id = i
			b.returnicon:SetScript("OnClick", Postal:GetModule("DoNotWant").Click);
			b.returnicon:SetScript("OnEnter", b:GetScript("OnEnter"));
			b.returnicon:SetScript("OnLeave", b:GetScript("OnLeave"));
		end
		-- For enabling after a disable
		b.returnicon:Show()
	end
end

-- ==================================================
--  Hide all buttons
-- ==================================================
function ExtMailUI_PostalDNW_RemoveReturnIcons()
    local i;
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
        local b = _G["MailItem" .. i .. "ExpireTime"].returnicon;
		if (b) then
            b:Hide();
		end
	end
end

-- ==================================================
--  Refresh buttons
-- ==================================================
function ExtMailUI_PostalDNW_RefreshReturnIcons()
    local i;
    local numItems, totalItems = GetInboxNumItems();
	for i = 1, ExtendedMailUI.TOTAL_ITEMS_VISIBLE do
        local mb = _G["MailItem" .. i .. "Button"];
        local b = _G["MailItem" .. i .. "ExpireTime"].returnicon;
        if (b) then
            if (i <= numItems) then
                b.id = mb.index;
                local f = InboxItemCanDelete(mb.index)
                b.texture:SetTexture(f and "Interface\\RaidFrame\\ReadyCheck-NotReady" or "Interface\\ChatFrame\\ChatFrameExpandArrow");
                b.tooltip = f and DELETE or MAIL_RETURN;
                b:Show();
            else
                b:Hide();
            end
        end
	end
end
