local L = LibStub("AceLocale-3.0"):GetLocale("ExtMailUI", true);
local STORE_NEW = {};
local CONFIG_SHOWN = false;

function ExtMailUI_ShowMainConfig()
    InterfaceOptionsFrame_OpenToCategory(ExtMailUIConfigContainer);
    InterfaceOptionsFrame_OpenToCategory(ExtMailUIConfigContainer);
end

--==================================================
-- Setting up the config frame
--==================================================
function ExtMailUI_Config_OnLoad(self)
    self.name = L["ADDON_TITLE"];
    self.okay = function(self) ExtMailUI_Config_Save(); end;
    self.cancel = function(self) ExtMailUI_Config_Cancel(); end;
    self.refresh = function(self) ExtMailUI_Config_Refresh(); end;
    self.default = function(self) ExtMailUI_Config_SetDefaults(); end;
    InterfaceOptions_AddCategory(self);

    ExtMailUIConfigTitle:SetText(string.format(L["VERSION_TEXT"], "|cffffffffv" .. ExtendedMailUI.Version));

    -- ********** General Options **********

    ExtMailUIConfig_GeneralContainerTitle:SetText(L["CONFIG_HEADING_GENERAL"]);

	ExtMailUIConfig_GeneralContainer_ShowLoadMsgText:SetText(L["OPTION_STARTUP_MESSAGE"]);
	ExtMailUIConfig_GeneralContainer_ShowLoadMsg.tooltip = L["OPTION_STARTUP_MESSAGE_TOOLTIP"];

	ExtMailUIConfig_GeneralContainer_HalfWidthText:SetText(L["OPTION_HALF_WIDTH"]);
	ExtMailUIConfig_GeneralContainer_HalfWidth.tooltip = L["OPTION_HALF_WIDTH_TOOLTIP"];
    
	ExtMailUIConfig_GeneralContainer_AutoLootText:SetText(L["OPTION_AUTO_LOOT"]);
	ExtMailUIConfig_GeneralContainer_AutoLoot.tooltip = L["OPTION_AUTO_LOOT_TOOLTIP"];
    
end

--==================================================
-- Handle when the configuration is opened
--==================================================
function ExtMailUI_Config_OnShow()
    if (CONFIG_SHOWN) then return; end
    ExtMailUI_Config_StoreCurrentSettings();
    CONFIG_SHOWN = true;
end

--==================================================
-- Sets the values of the controls to
-- reflect currently loaded settings
--==================================================
function ExtMailUI_Config_Refresh()
    ExtMailUI_Config_OnShow();
	ExtMailUIConfig_GeneralContainer_ShowLoadMsg:SetChecked(STORE_NEW.loadMessage);
	ExtMailUIConfig_GeneralContainer_HalfWidth:SetChecked(STORE_NEW.halfWidthMode);
	ExtMailUIConfig_GeneralContainer_AutoLoot:SetChecked(STORE_NEW.enableAutoLoot);
end

--==================================================
-- Store current settings
--==================================================
function ExtMailUI_Config_StoreCurrentSettings()
    STORE_NEW = {
        loadMessage = EXTMAILUI_DATA['config'].loadMessage or false,
        enableAutoLoot = EXTMAILUI_DATA['config'].enableAutoLoot or false,
        halfWidthMode = EXTMAILUI_DATA['config'].halfWidthMode or false,
    };
end

--==================================================
-- Closing the config window - copy new settings to
-- main variables and update half-width mode state
--==================================================
function ExtMailUI_Config_Save()
    EXTMAILUI_DATA['config'].loadMessage = STORE_NEW.loadMessage;
    EXTMAILUI_DATA['config'].halfWidthMode = STORE_NEW.halfWidthMode;
    EXTMAILUI_DATA['config'].enableAutoLoot = STORE_NEW.enableAutoLoot;
    ExtMailUI_SetHalfMode(EXTMAILUI_DATA['config'].halfWidthMode);
    CONFIG_SHOWN = false;
end

--==================================================
-- Handle clicking the Cancel button
--==================================================
function ExtMailUI_Config_Cancel()
    CONFIG_SHOWN = false;
end

--==================================================
-- Handler for checking/unchecking checkbox(es)
--==================================================
function ExtMailUI_Config_CheckBox_OnClick(self, id)
	if (id == 1) then
		if (self:GetChecked()) then
            STORE_NEW.loadMessage = true;
		else
		    STORE_NEW.loadMessage = false;
		end
    elseif (id == 2) then
		if (self:GetChecked()) then
		    STORE_NEW.enableAutoLoot = true;
		else
		    STORE_NEW.enableAutoLoot = false;
		end
    elseif (id == 3) then
		if (self:GetChecked()) then
		    STORE_NEW.halfWidthMode = true;
		else
		    STORE_NEW.halfWidthMode = false;
		end
	end
end

--==================================================
-- Handler for mousing over options
-- on the config window
--==================================================
function ExtMailUI_Config_Option_OnEnter(self)
	if (self.tooltip) then
        GameTooltip:SetOwner(self, "ANCHOR_NONE");
		GameTooltip:SetPoint("TOPLEFT", self:GetName(), "BOTTOMLEFT", -10, -4);
        GameTooltip:SetText(self.tooltip, 1, 1, 1);
        GameTooltip:Show();
	end
end

--==================================================
-- Moving the mouse away from config options
--==================================================
function ExtMailUI_Config_Option_OnLeave(self)
	GameTooltip:Hide();
end

--==================================================
-- Handler for changing slider(s) [for future use]
--==================================================
function ExtMailUI_Config_Slider_OnValueChanged(self, id)

    if (id == 1) then
    end

end

--==================================================
-- Slider text update for value changes
--==================================================
function ExtMailUI_Config_Slider_UpdateText(slider, text)
    _G[slider:GetName() .. "Text1"]:SetText(text);
end
